<?php

use \Bitrix\Sale\Internals\BusinessValueTable;
use \Bitrix\Sale\Order;
use Lootils\Uuid\Exception;

class DeloProcessor
{
    const QRCODE_STATUS_DEFAULT = 'NTST';
    const QRCODE_STATUS_SUCCESS = 'ACWP';
    const QRCODE_STATUS_REJECTED = 'RJCT';
    const PREFIX = 'DELOBANK';

    /**
     * Возвращает заказ в виде массива
     * @param int $orderId
     * @return array
     * @throws \Bitrix\Main\ArgumentNullException
     * @throws \Bitrix\Main\ObjectNotFoundException
     */
    public static function findOrder($orderId)
    {
        $orderArr = [
            'ID'                => 0,
            'PRICE'             => 0,
            'ACCOUNT_NUMBER'    => 0,
            'PAY_SYSTEM_ID'     => [],
            'PAY_SYSTEM_ID_NEW' => [],
            'PERSON_TYPE_ID'    => 0,
        ];
        /** @var Order $order */
        $order = Order::load($orderId);
        if (empty($order)) {
            $CSaleOrder = new CSaleOrder();
            $dbOrder = $CSaleOrder->GetList(['ID' => 'ASC'], ['ACCOUNT_NUMBER' => $orderId]);
            $order = $dbOrder->GetNext();
            if (!empty($order)) {
                $orderArr = [
                    'ID'                => (int)$order['ID'],
                    'PRICE'             => $order['PRICE'],
                    'ACCOUNT_NUMBER'    => $order['ACCOUNT_NUMBER'],
                    'PAY_SYSTEM_ID'     => $order['PAY_SYSTEM_ID'],
                    'PERSON_TYPE_ID'    => $order['PERSON_TYPE_ID'],
                ];
            }
        } else {
            $orderArr = [
                'ID'                => (int)$order->getId(),
                'PRICE'             => $order->getPrice(),
                'ACCOUNT_NUMBER'    => 0,
                'PAY_SYSTEM_ID'     => current($order->getPaymentSystemId()),
                'PERSON_TYPE_ID'    => $order->getPersonTypeId(),
            ];
        }
        return $orderArr;
    }

    /**
     * Изменяет статус заказа.
     * @param int $orderId
     * @param string $status
     * @param int $amount
     * @param string $authCode
     * @param array $settings
     * @param string $errorCode
     * @return bool|int
     * @throws \Bitrix\Main\ArgumentNullException
     * @throws \Bitrix\Main\ArgumentOutOfRangeException
     */
    public static function setStatusCode($orderId, $status, $amount, $authCode, $settings, $errorCode)
    {
        $CDatabase = new CDatabase();
        $CSaleOrder = new CSaleOrder();
        $date = date($CDatabase->DateFormatToPHP(CLang::GetDateFormat('FULL', LANG)));
        // По умолчанию неудачный платеж
        $arFields = [
            'PAYED'             => 'N',
            'PS_STATUS'         => 'N',
            'PS_STATUS_MESSAGE' => self::getMessageErrorByCode($errorCode),
            'PS_RESPONSE_DATE'  => $date,
            'DATE_PAYED'        => $date,
            'PS_SUM'            => $amount,
            'PS_STATUS_CODE'    => (string)$status,
        ];
        if ($status == 1) {
            // Удачный платеж
            $arFields['PAYED'] = 'Y';
            $arFields['PS_STATUS'] = 'Y';
            $arFields['PS_STATUS_MESSAGE'] = 'Оплата прошла успешно.';
            $arFields['PS_STATUS_DESCRIPTION'] = self::clearData($authCode);
            if (!empty($settings['AFTER_STATUS'])) {
                $arFields['STATUS_ID'] = $settings['AFTER_STATUS'];
                $arFields['DATE_STATUS'] = $date;
            }
        }
        // Изменяет статус заказа.
        return $CSaleOrder->Update($orderId, $arFields);
    }

    /**
     * @param int $orderId id заказа
     * @param string $xmlResponse xml ответа
     * @throws Exception
     */
    public static function logRecord($orderId, $xmlResponse)
    {
        global $DB;
        $serverProtocol = self::getServerProtocol();
        $session_id = isset($_GET['trx_id']) ? $_GET['trx_id'] : '';
        $request = json_encode($_GET);
        $timestamp = new DateTime();
        $event = [
            'action_to'     => 'BITRIX',
            'action_name'   => self::isCheckPaymentRequest() ? 'PAY_CHECK' : 'PAY_REG',
            'request_data'  => str_replace('&quot;', '', htmlspecialchars($request, ENT_QUOTES)),
            'response_data' => $xmlResponse,
            'order_id'      => $orderId,
            'session_id'    => $session_id,
            'url'           => $serverProtocol . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'],
            'registered_ts' => $timestamp->getTimestamp(),
            'ip'            => $_SERVER['REMOTE_ADDR']
        ];
        $DB->Query(
            "INSERT INTO b_delobank_log (`".implode('`,`', array_keys($event))."`) VALUES ('".implode("', '", $event)."');"
            , false
            , 'File: ' . __FILE__ . '<br>Line: ' . __LINE__
        );
    }

    /**
     * Проверка существования сертификата эквайера. Может вызываться на обоих фазах проведения платежа.
     * Убедимся, что пользователь залил серификат по фтп на сайт и корректно прописал путь до него в настройках платежного модуля
     * @param array $paymentSettings
     * @throws Exception
     * @return bool
     */
    public static function checkCertificate($paymentSettings)
    {
        if ($paymentSettings['CHECK_SIGNATURE'] === 'N') {
            return true;
        }
        $certFile = $_SERVER['DOCUMENT_ROOT'] . $paymentSettings['CERT_PATH'];
        if (!file_exists($certFile) || !is_file($certFile)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Проверка подписи (ЭЦП) в запросе от эквайера
     * @param array $paymentSettings настройки
     * @param int $orderId id заказа
     * @return bool true - если проверка пройдена и false если не пройдена по какой-либо причине
     */
    public static function checkSignature($paymentSettings, $orderId)
    {
        if ($paymentSettings['CHECK_SIGNATURE'] === 'N') {
            // Если проверка ЦП выключена, параметра signature не будет, проверку ЦП не делаем всегда ок
            return true;
        }
        $signature = base64_decode($_GET['signature']);
        $data = self::getDataFromUrl();
        $certFile = realpath($_SERVER['DOCUMENT_ROOT'] . $paymentSettings['CERT_PATH']);
        // Получаем из сертификата публичный ключ для проверки
        $public_key = openssl_pkey_get_public('file://' . $certFile);
        if ($public_key === false) {
            return false;
        }
        // Проверяем подпись и освобождаем ключ
        $signature_check = openssl_verify($data, $signature, $public_key, OPENSSL_ALGO_SHA1);
        openssl_free_key($public_key);
        if ($signature_check !== 1) {
            return false;
        }
        return true;
    }

    /**
     * Выделяет подписанные данные из запроса
     * @return string часть url-а c доменом до &signature=
     */
    private static function getDataFromUrl()
    {
        $url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? 'https' : 'http') . "://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
        $pos = strpos($url, '&signature=');
        return substr($url, 0, $pos);
    }

    /**
     * Положительный ответ на проверку возможности проведения платежа
     * @param array $order массив с данными заказа
     * @param array $paymentSettings настройки
     * @throws Exception
     */
    public static function paymentAvailabilityPositiveResponse($order, $paymentSettings)
    {
        $sxe = new SimpleXMLElement("<payment-avail-response></payment-avail-response>");
        $result = $sxe->addChild('result');
        $result->addChild('code', 1);
        $result->addChild('desc', 'OK');
        $purchase = $sxe->addChild('purchase');
        $purchase->addChild('shortDesc', 'Заказ ' . $order['ID']);
        $purchase->addChild('longDesc', 'Заказ ' . $order['ID']);
        $accountAmount = $purchase->addChild('account-amount');
        $accountAmount->addChild('id', $paymentSettings['ACCOUNT_PCID']);
        $accountAmount->addChild('amount', self::rubToPennies($order['PRICE']));
        $accountAmount->addChild('currency', $paymentSettings['CURRENCY']);
        $accountAmount->addChild('exponent', '2');
        $responseXml = $sxe->asXML();
        self::logRecord($order['ID'], $responseXml);
        self::sendResponse($responseXml);
    }

    /**
     * @param string $param
     * @return string
     */
    public static function clearData($param)
    {
        $search = ['"\'"si', '"\""si', '"\`"si'];
        return preg_replace($search, '', trim(strip_tags($param)));
    }

    /**
     * @param int $paymentSystemId
     * @param int $personTypeId
     * @param int $method 1 - ввод данных карточки 2 - QR-код
     * @return array
     */
    static public function getPaymentSettings($paymentSystemId, $personTypeId, $method = 1)
    {
        $paymentSettings = [];
        if (empty($paymentSystemId)) {
            return $paymentSettings;
        }
        if ($method == 1) {
            $fields = [
                'SHOP_PCID',
                'ACCOUNT_PCID',
                'CURRENCY',
                'CERT_PATH',
                'SUCCESS_URL',
                'DECLINE_URL',
                'START_PAYMENT_URL',
                'LANG_CODE',
                'CHECK_SIGNATURE',
                'AFTER_STATUS',
            ];
        } else {
            $fields = [
                'AFTER_STATUS',
                'MERCHANT_ID_QRCODE',
                'LOGIN_QRCODE',
                'PASSWORD_QRCODE',
                'URL_QRCODE',
            ];
        }
        $options = [
            'select' => ['CODE_KEY', 'PROVIDER_VALUE'],
            'filter' => [
                '=CODE_KEY' => $fields,
                '=CONSUMER_KEY'   => 'PAYSYSTEM_' . $paymentSystemId,
                '=PERSON_TYPE_ID' => 0,
            ],
        ];
        if (!empty($personTypeId)) {
            // Проверяем есть ли настройка для указанного типа плательщика $personTypeId (1 физ.лицо или 2 юр.лицо)
            $options['filter']['=PERSON_TYPE_ID'] = $personTypeId;
            try {
                $result = BusinessValueTable::getList($options);
                while ($row = $result->fetch()) {
                    $paymentSettings[$row['CODE_KEY']] = $row['PROVIDER_VALUE'];
                }
            } catch (\Exception $ex) {
                $paymentSettings = [];
            }
        }
        /**
         * @hotfix
         * Проверяем есть ли все настройки для указанного типа плательщика, если нету то пробуем взять настройки с типом плательщика = 0
         * PERSON_TYPE_ID = 1 - настройки для физ лица
         * PERSON_TYPE_ID = 2 - настройки для юр лица
         * PERSON_TYPE_ID = 0 - настройки для неизвестного
         */
        $needDefaultValues = [];
        foreach ($fields as $field) {
            if (empty($paymentSettings[$field])) {
                $needDefaultValues[] = $field;
            }
        }
        if (!empty($needDefaultValues)) {
            // Добираем нужные настройки из неизвестного
            $options['filter']['=PERSON_TYPE_ID'] = 0;
            $options['filter']['=CODE_KEY'] = $needDefaultValues;
            try {
                $result = BusinessValueTable::getList($options);
                while ($row = $result->fetch()) {
                    $paymentSettings[ $row['CODE_KEY'] ] = $row['PROVIDER_VALUE'];
                }
            } catch (\Exception $ex) {
                $paymentSettings = [];
            }
        }
        // По умолчанию
        $needDefaultValues = [];
        foreach ($fields as $field) {
            if (empty($paymentSettings[$field])) {
                $needDefaultValues[] = $field;
            }
        }
        if (!empty($needDefaultValues)) {
            // Добираем нужные настройки из "по умолчанию"
            $options['filter']['=PERSON_TYPE_ID'] = 0;
            $options['filter']['=CODE_KEY'] = $needDefaultValues;
            $options['filter']['=CONSUMER_KEY'] = 'COMMON';
            try {
                $result = BusinessValueTable::getList($options);
                while ($row = $result->fetch()) {
                    $paymentSettings[ $row['CODE_KEY'] ] = $row['PROVIDER_VALUE'];
                }
            } catch (\Exception $ex) {
                $paymentSettings = [];
            }
        }
        return $paymentSettings;
    }

    /**
     * @return string
     */
    static private function getServerProtocol()
    {
        $serverProtocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']=='on') ? 'HTTPS' : 'HTTP';
        return $serverProtocol;
    }

    /**
     * Определяет по параметрам запроса, запрос ли это на проверку платежа
     * @return bool
     */
    static public function isCheckPaymentRequest()
    {
        if (!isset($_GET['result_code'])) {
            return true;
        }
        return false;
    }

    /**
     * Возвращает кусок xml для ответа
     * @param int $code 1 - ок, 2 - что-то пошло не так
     * @param string $desc описание
     * @return string
     */
    static public function setResponse($code, $desc)
    {
        if (empty($code) || empty($desc)) {
            return '';
        }
        if (self::isCheckPaymentRequest()) {
            $xml = '<payment-avail-response>';
        } else {
            $xml = '<register-payment-response>';
        }
        $xml .= '<result><code>' . $code . '</code><desc><![CDATA[' . $desc . ']]></desc></result>';
        if (self::isCheckPaymentRequest()) {
            $xml .= '</payment-avail-response>';
        } else {
            $xml .= '</register-payment-response>';
        }
        return $xml;
    }

    /**
     * Выставляет правильные заголовки и выплевывает ответ
     * @param string $xml xml ответа
     */
    static public function sendResponse($xml)
    {
        header('Content-type: application/xml');
        exit($xml);
    }

    /**
     * Возвращает сообщение ошибки ее коду
     * @param string $code
     * @return string
     */
    static private function getMessageErrorByCode($code)
    {
        $errors = [
            '-2' => 'Отказ в проведении платежа',
            '-3' => 'Ошибка при проведении платежа',
            '-4' => 'Успешный платеж отменен по инициативе магазина на этапе фиксации платежа',
            '2'  => 'Магазин отказался проводить платеж на этапе проверки',
            '4'  => 'Произошла ошибка при взаимодействии с магазином на этапе проверки платежа',
            '21' => 'Проверка платежа в магазине не выполнялась. Возможно магазин, или биллинг были заблокированы',
            '53' => 'Транзакция завершилась по таймауту, так как пользователь по каким-то причинам отказался продолжать платеж',
        ];
        if (isset($errors[$code])) {
            return $errors[$code];
        }
        return 'Ошибка проведения платежа с кодом ошибки: ' . $code;
    }

    /**
     * Преобразуем рубли в копейки
     * @param string $amount
     * @return int
     */
    static public function rubToPennies($amount)
    {
        $amount = number_format($amount, 2, '', '');
        return (int)$amount;
    }

    /**
     * Преобразуем копейки в рубли
     * @param string $amount
     * @return string
     */
    static public function penniesToRub($amount)
    {
        return number_format($amount / 100, 2, '.', '');
    }

    /**
     * Генерирует messageId для QR-кода и запоминает его в БД
     * @param string $merchantIdQrcode
     * @param int $orderId
     * @return string uid
     */
    public static function createDataQRCode($merchantIdQrcode, $orderId)
    {
        global $DB;
        // Подлючаем библиотеку генерации UUID
        include_once $_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/sale/payment/delobankqrcode/lib/Lootils/Uuid/Exception.php';
        include_once $_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/sale/payment/delobankqrcode/lib/Lootils/Uuid/Uuid.php';
        try {
            $messageUid = \Lootils\Uuid\Uuid::createV5($merchantIdQrcode, $orderId);
        } catch (Lootils\Uuid\Exception $exception) {
            // Если на основании $merchantIdQrcode и $orderId сформировать не получилось тогда генерируем случайный
            $messageUid = \Lootils\Uuid\Uuid::createV4();
        }
        $now = time();
        $q = 'INSERT INTO `b_delobank_order_qrcode` (
                `order_id`,
                `message_id`,
                `date_create`
            ) VALUE (
                ' . (int)$orderId . ',
                "' . (string)$messageUid . '",
                ' . $now . '
            )
            ON DUPLICATE KEY UPDATE `message_id` = "' . (string)$messageUid . '", `date_create` = ' . $now;
        $DB->Query($q);
        return (string)$messageUid;
    }

    /**
     * Возвращает информацию по QR-коду
     * @param int $orderId
     * @return array|null
     */
    public static function getDataQRCode($orderId)
    {
        global $DB;
        /** @var CDBResult $result */
        $result = $DB->Query('SELECT
                `order_id`,
                `message_id`,
                `status`,
                `qrc_id`,
                `qrc_link`,
                `qrc_extension`
            FROM `b_delobank_order_qrcode`
            WHERE `order_id` = ' . (int)$orderId);
        $message = $result->Fetch();
        if (empty($message['message_id'])) {
            return null;
        }
        return $message;
    }

    /**
     * Выставляет переменные для взаимодействия с spb.skbbank.ru в сессию
     * @param array $paymentSettings
     * @param array $dataQRCode
     * @param array $order
     */
    public static function setDataQRCodeInSession($paymentSettings, $dataQRCode, $order)
    {
        $_SESSION['delobank_login_qrcode'] = $paymentSettings['LOGIN_QRCODE'];
        $_SESSION['delobank_password_qrcode'] = $paymentSettings['PASSWORD_QRCODE'];
        $_SESSION['delobank_merchant_id_qrcode'] = $paymentSettings['MERCHANT_ID_QRCODE'];
        $_SESSION['delobank_message_id_qrcode'] = $dataQRCode['message_id'];
        $_SESSION['delobank_url_qrcode'] = $paymentSettings['URL_QRCODE'];
        $_SESSION['delobank_order_id_qrcode'] = (int)$order['ID'];
        $_SESSION['delobank_qrc_id_qrcode'] = $dataQRCode['qrc_id'];
    }

    /**
     * Делает запрос к СБП за QR-кодом
     * @param array $paymentSettings
     * @param string $messageUid uid сооб
     * @param array $order
     * @return array|null
     */
    public static function queryRegisterQRCode($paymentSettings, $messageUid, $order)
    {
        global $DB;
        $responseHeaders = [];
        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL            => $paymentSettings['URL_QRCODE'] . '/register',
            CURLOPT_HTTPHEADER     => ['Content-Type: application/json'],
            CURLOPT_RETURNTRANSFER => 1, // чтобы возвращал в переменную, а не делал echo
            CURLOPT_FOLLOWLOCATION => 1, // чтобы делал все переходы
            CURLOPT_TIMEOUT        => 10,
            CURLOPT_USERAGENT      => 'DeloBank Payment Module CMS Bitrix from https://' . $_SERVER['HTTP_HOST'],
            CURLOPT_POST           => 1,
            CURLOPT_POSTFIELDS     => json_encode([
                'messageId'      => $messageUid,
                'merchantId'     => $paymentSettings['MERCHANT_ID_QRCODE'],
                'amount'         => (string)self::rubToPennies($order['PRICE']),
                'paymentPurpose' => 'Заказ №' . $order['ID'] . ' https://' . $_SERVER['HTTP_HOST'] . '/',
                'returnImage'    => false,
            ]),
            CURLOPT_HTTPAUTH       => CURLAUTH_BASIC,
            CURLOPT_USERPWD        => $paymentSettings['LOGIN_QRCODE'] . ':' . $paymentSettings['PASSWORD_QRCODE'],
            CURLOPT_SSL_VERIFYPEER => isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 0 : 1,
            CURLOPT_SSL_VERIFYHOST => isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 0 : 2,
            // Ниже функция помогающая получить заголовки ответа
            CURLOPT_HEADERFUNCTION => function($curl, $header) use (&$responseHeaders) {
                $len = strlen($header);
                $header = explode(':', $header, 2);
                if (count($header) < 2) return $len;
                $name = strtolower(trim($header[0]));
                if (!array_key_exists($name, $responseHeaders)) $responseHeaders[$name] = [trim($header[1])];
                else $responseHeaders[$name][] = trim($header[1]);
                return $len;
            },
        ]);
        $result = curl_exec($curl);
        if (!$result) {
            // Какая-либо ошибка в запросе
            self::inLog('Ошибка в запросе до СБП на регистрацию QR-кода: ' . curl_error($curl));
            return null;
        }
        // Парсим json ответа
        $result = json_decode($result, true);
        if (($result['errCode'] == '0') && ($result['status'] == 'CREATED') && !empty($result['qrcId'])) {
            // Если нет ошибок и QR-код сгенерирован
            $qrcId = $result['qrcId'];
            $qrcLink = $result['payload'];
            // Получаем картинку QR-кода
            $responseHeaders = [];
            curl_setopt_array($curl, [
                CURLOPT_URL        => $paymentSettings['URL_QRCODE'] . '/getpayload',
                CURLOPT_POSTFIELDS => json_encode([
                    'messageId'   => $messageUid,
                    'qrcId'       => $qrcId,
                    'returnImage' => 'true',
                ]),
            ]);
            $result = curl_exec($curl);
            if (!$result) {
                self::inLog('Ошибка в запросе до СБП на повторное получение информации о QR-коде: ' . curl_error($curl));
                return null;
            } else {
                $ext = str_replace('image/', '', $responseHeaders['content-type'][0]);
                $imgPath = $_SERVER['DOCUMENT_ROOT'] . '/personal/order/payment/delobankqrcode/img/' . $qrcId . '.' . $ext;
                if (file_put_contents($imgPath, $result)) {
                    $q = 'UPDATE b_delobank_order_qrcode SET
                            qrc_id = "' . $qrcId . '",
                            qrc_link = "' . $qrcLink . '",
                            qrc_extension = "' . $ext . '"
                        WHERE order_id = ' . (int)$order['ID'];
                    $DB->query($q);
                    return [
                        'order_id'      => $order['ID'],
                        'message_id'    => $messageUid,
                        'status'        => self::QRCODE_STATUS_DEFAULT,
                        'qrc_id'        => $qrcId,
                        'qrc_link'      => $qrcLink,
                        'qrc_extension' => $ext,
                    ];
                } else {
                    self::inLog('Не получается сохранить изображение QR-кода');
                    return null;
                }
            }
        } else {
            self::inLog('Пустой ответ на запрос до СБП на регистрацию QR-кода. URL: ' . $paymentSettings['URL_QRCODE'] . '/register');
            return null;
        }
    }

    /**
     * Пишет в журнал событий
     * @param string $message сообщение
     * @param string $type может принимать значения SECURITY, ERROR, WARNING, INFO, DEBUG
     * @param int $itemId id пользователя или другого объекта
     */
    public static function inLog($message, $type = 'ERROR', $itemId = 123)
    {
        CEventLog::Add([
            'SEVERITY'      => $type,
            'AUDIT_TYPE_ID' => 'DELOBANK PAYMENT',
            'MODULE_ID'     => 'main',
            'ITEM_ID'       => $itemId,
            'DESCRIPTION'   => $message,
        ]);
    }

    /**
     * Делает 303 редирект
     * @param string $url
     */
    public static function redirect($url)
    {
        header('HTTP/1.1 303 See Other');
        header('Location: ' . $url);
        exit();
    }

    /**
     * Делает запрос к СБП за результатом, оплачен ли заказ по QR-коду
     * @param string $url
     * @param string $login
     * @param string $password
     * @param string $messageUid
     * @param array $qrc
     * @return null|array
     */
    public static function queryGetPaymentStatus($url, $login, $password, $messageUid, $qrc)
    {
        $responseHeaders = [];
        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL            => $url . '/getpaymentsstatus',
            CURLOPT_HTTPHEADER     => ['Content-Type: application/json'],
            CURLOPT_RETURNTRANSFER => 1, // чтобы возвращал в переменную, а не делал echo
            CURLOPT_FOLLOWLOCATION => 1, // чтобы делал все переходы
            CURLOPT_TIMEOUT        => 10,
            CURLOPT_USERAGENT      => 'DeloBank Payment Module CMS Bitrix from https://' . $_SERVER['HTTP_HOST'],
            CURLOPT_POST           => 1,
            CURLOPT_POSTFIELDS     => json_encode([
                'messageId' => $messageUid,
                'qrcIds'    => $qrc,
            ]),
            CURLOPT_HTTPAUTH       => CURLAUTH_BASIC,
            CURLOPT_USERPWD        => $login . ':' . $password,
            CURLOPT_SSL_VERIFYPEER => isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 0 : 1,
            CURLOPT_SSL_VERIFYHOST => isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 0 : 2,
            // Ниже функция помогающая получить заголовки ответа
            CURLOPT_HEADERFUNCTION => function($curl, $header) use (&$responseHeaders) {
                $len = strlen($header);
                $header = explode(':', $header, 2);
                if (count($header) < 2) return $len;
                $name = strtolower(trim($header[0]));
                if (!array_key_exists($name, $responseHeaders)) $responseHeaders[$name] = [trim($header[1])];
                else $responseHeaders[$name][] = trim($header[1]);
                return $len;
            },
        ]);
        $result = curl_exec($curl);
        if (!$result) {
            // Какая-либо ошибка в запросе
            self::inLog('Ошибка запроса к сервису быстрых платежей: ' . curl_error($curl));
            return null;
        }
        $resultArray = json_decode($result, true);
        // Для того чтобы показать как будет при успешном платеже
        // $resultArray['payments'][0]['status'] = self::QRCODE_STATUS_SUCCESS;
        return $resultArray;
    }
}
